require 'agape-red-recipes/modules/bundler'
require 'agape-red-recipes/modules/gemfile'
require 'agape-red-recipes/modules/gems'

class RecipeGenerator < Rails::Generators::Base
  include AgapeRedRecipes::Bundler
  include AgapeRedRecipes::Gemfile
  include AgapeRedRecipes::Gems

  argument :recipe_name, type: :string, default: 'list'
  argument :settings, type: :array, default: []

  def generate_recipe
    the_options = {}
    settings.each{|s| data = s.split(':'); the_options[data[0]] = data[1] }

    if recipe_name == 'list'
      list_repos
    else
      apply_recipe recipe_name, the_options
    end
  end

  private
  def list_repos
    require 'net/http'
    require 'uri'

    def open(url)
      Net::HTTP.get(URI.parse(url))
    end

    page_content = JSON.parse(open('https://api.github.com/users/AgapeRedRecipes/repos'))
    repos = page_content.map{|repo| repo['name']}.select{|repo| repo != 'agape-red-recipes'}
    puts "You can apply the following recipes using `rails g recipe [recipe_name] [options]`"
    repos.each{|r| puts "    #{r.downcase}" }
  end

  def apply_recipe the_recipe, the_options
    if the_recipe.include?('.')
      RecipeGenerator.source_root(Rails.root)
      apply the_recipe, the_options
    elsif the_recipe.starts_with?('http')
      apply the_recipe, the_options
    else
      apply "https://raw.github.com/AgapeRedRecipes/#{the_recipe}/master/recipe.rb", the_options
    end
  end

end
